;;;; posix-utils.scm
;;;; Kon Lovett, Nov '10
;;;; Kon Lovett, Feb '18

;; Issues
;;

(module posix-utils

(;export
  ;
  string-trim-eol software-eol-string
  ;
  qs* qs-literal qs-string qs-argument
  ;
  get-commandline-result
  get-command-result
  get-echo-result
  ;
  environment-string environment-variables->environment-list
  ;
  get-shell-variable
  ;
  environment-variable-bound?
  environment-variable-true?
  register-environment-variable-feature!
  ;
  shell-variable-bound?
  shell-variable-true?
  register-shell-variable-feature! )

(import scheme chicken)
(use
  (only (srfi 1) member)
  (only data-structures conc string-intersperse o identity ->string)
  (only posix process process-wait)
  (only utils qs read-all)
  (only (srfi 13) string-trim-both string-index string-take string-null? string-downcase)
  (only (srfi 14) list->char-set)
  (only check-errors check-string) )

;;

(define (qs-delimiter obj)
  (if (string? obj) obj (string obj)) )

;;

(define-constant *SHELL-TRUE-VALUES* '("y" "yes" "1"))

(: shell-value-bound? ((or boolean string) -> (or boolean string)))
(define (shell-value-bound? varval)
  (and-let* (
    (str varval)
    ((not (string-null? str))) )
    str ) )

(: shell-value-true? ((or boolean string) (or boolean (list-of string)) -> boolean))
(define (shell-value-true? varval truedat)
  (and-let* ((str varval))
    (let* (
      (truedat (or truedat *SHELL-TRUE-VALUES*))
      (str (string-trim-both str))
      (str (string-downcase str)) )
      (member str truedat string=?) ) ) )

(: register-shell-value-feature! ((or boolean string) procedure -> (or boolean symbol)))
(define (register-shell-value-feature! varval filter)
  (and-let* (
    (str (shell-value-bound? varval))
    (str (filter str)) )
    (let (
      (varsym (string->symbol str)) )
      (register-feature! varsym)
      varsym ) ) )

;;

(define software-eol-sequence
  (cond-expand
    (windows
     '(#\return #\newline))
    (unix
     '(#\newline))
    (else
     '(#\return)) ) )

(define char-set:software-eol (list->char-set software-eol-sequence))

(: string-trim-string string)
;
(define software-eol-string (list->string software-eol-sequence))

(define software-eol-length (string-length software-eol-string))

(: string-trim-eol (string --> string))
;
(define (string-trim-eol str)
  (let* (
    (end (string-length str))
    (start (fx- end software-eol-length))
    (index
      (and
        (fx<= 0 start) (fx< start end)
        ;order of multi-char sequences is not imposed !
        (string-index str char-set:software-eol start end))) )
    (if (not index)
      str
      (string-take str index) ) ) )

;;

(define-constant DOUBLE-QUOTE "\"" #;"\"")  ;editor unbalanced
(define-constant SINGLE-QUOTE "\'")
(define-constant BACK-QUOTE "`")

(: qs* (string string #!optional (or string char) -> string))
;
(define (qs* str #!optional (delim DOUBLE-QUOTE))
  (let ((delim (qs-delimiter delim)))
    (string-append delim str delim) ) )

(: qs-literal (* -> string))
;
(define (qs-literal item)
  (qs-argument item #t SINGLE-QUOTE) )

(: qs-string (* -> string))
;
(define (qs-string item)
  (qs-argument item #t DOUBLE-QUOTE) )

(: qs-argument (* #!optional boolean (or string char) -> string))
;
(define (qs-argument item #!optional literal? (delim DOUBLE-QUOTE))
  (cond
    ;
    ;' item '
    ((and (not literal?) (pair? item) (eq? 'quote (car item)))
      (qs-argument (cadr item) #t SINGLE-QUOTE) )
    ;
    ;` item ... ` : quasi-interpret qs-argument
    ;   , - " item "
    ;   ,@ - " ->string(item) ... "
    ((and (not literal?) (pair? item) (eq? 'quasiquote (car item)))
      (qs-argument (qs-quasiquote (cadr item)) #t BACK-QUOTE) )
    ;
    ;< item >
    ((string? item)
      (qs* item delim) )
    ;
    ;< string(item) >
    (else
      (qs* (->string item) delim)) ) )

(: qs-quasiquote (* --> string))
;
(define (qs-quasiquote item)
  (cond
    ;
    ;" item "
    ((and (pair? item) (eq? 'unquote (car item)))
      (qs-unquote (cadr item)) )
    ;
    ;" item ... "
    ((and (pair? item) (eq? 'unquote-splicing (car item)))
      (qs-unquote-splicing (cadr item)) )
    ;
    ;' item '
    (else
      (qs-argument item #t SINGLE-QUOTE) ) ) )

(: qs-unquote-splicing (* --> string))
;
(define (qs-unquote-splicing item)
  (qs-unquote item #t) )

(: qs-unquoted (string --> string))
;
(define qs-unquoted (o ->string eval))

(: qs-unquote (* #!optional boolean --> string))
;
(define (qs-unquote item #!optional splicing?)
  (let* (
    (evaled
      (if (and splicing? (list? item))
        (string-intersperse (map qs-unquoted item) " ")
        (qs-unquoted item) ) ) )
    (qs-argument evaled #t DOUBLE-QUOTE) ) )

;;

(: environment-string (pair -> string))
;
(define (environment-string pare)
  (string-append (car pare) "=" (->string (cdr pare))) )

(: environment-variables->environment-list (list -> list))
;
(define (environment-variables->environment-list al)
  (map environment-string al) )

;;

(: environment-variable-bound? (string -> (or boolean string)))
;
(define (environment-variable-bound? nam)
  (shell-value-bound? (get-environment-variable nam)) )

(: environment-variable-true? (string #!optional (or boolean (list-of string)) -> boolean))
;
(define (environment-variable-true? nam #!optional truedat)
  (shell-value-true? (get-environment-variable nam) truedat) )

(: register-environment-variable-feature! (string -> (or boolean symbol)))
;
(define (register-environment-variable-feature! nam #!optional (filter identity))
  (register-shell-value-feature! (get-environment-variable nam) filter) )

;;

(: get-commandline-result (string string -> string))
;
(define (get-commandline-result cmnd line)
  (let ((in (void)) (out (void)) (pid (void)))
    (dynamic-wind
      (lambda ()
        (let ((cmndline (string-append (qs cmnd) " " line)))
          (set!-values (in out pid) (process cmndline)) ) )
      (lambda ()
        (read-all in) )
      (lambda ()
        (close-input-port in)
        (close-output-port out) ) ) ) )

(: get-command-result (string #!rest -> string))
;
(define (get-command-result cmnd . args)
  (let ((line (string-intersperse (map qs-argument args) " ")))
    (get-commandline-result cmnd line) ) )

(: get-echo-result (#!rest -> string))
;
(define (get-echo-result . exps)
  (let ((res (apply get-command-result "echo" exps)))
    ;remove echo eol
    (string-trim-eol res) ) )

;;

(: get-shell-variable (string -> string))
;
(define (get-shell-variable name)
  (get-echo-result (string-append "$" name)) )

(: shell-variable-bound? (string -> (or boolean string)))
;
(define (shell-variable-bound? name)
  (shell-value-bound? (get-shell-variable name)) )

(: shell-variable-true? (string #!optional (or boolean (list-of string)) -> boolean))
;
(define (shell-variable-true? name #!optional truedat)
  (shell-value-true? (get-shell-variable name) truedat) )

(: register-shell-variable-feature! (string -> (or boolean symbol)))
;
(define (register-shell-variable-feature! name #!optional (filter identity))
  (register-shell-value-feature! (get-shell-variable name) filter) )

;;;

) ;module posix-utils
